#include "types.h"

#define TAB_WIDTH  4
#define CGA_SCREEN ((volatile char*)0xb8000)


static inline void kclearline(const unsigned line)
{
	for(int i=1; i<80*2; i+=2)
		CGA_SCREEN[line*80*2+i] = 0;
}


int kputc(const char c)
{
	static unsigned line = 0, col = 0;
	
	if( c == '\n' ) col = 80*2;
	else if( c == '\r' ) col = 0;
	else if( c == '\t' ) col += TAB_WIDTH*2;
	else
	{
		CGA_SCREEN[line*80*2+col++] = c;
		CGA_SCREEN[line*80*2+col++] = 15;
	}
	if(col >= (80*2))
	{
		line = (line+1)%25;
		col = 0;
		kclearline(line);
	}
	return c;
}


static void kprintu(uintdivmax_t n, unsigned base)
{
	char buf[UINTDIVMAX_BITS];
	int i = 0;
	
	if( base > 16 || base < 2 ) base = 10;
	
	do {
		unsigned r = n%base;
		buf[i++] = (r < 10 ? '0' : 'a'-10) + r;
		n /= base;
	} while( n > 0 );
	
	while(--i>=0)
		kputc(buf[i]);
}


static void kprintd(intdivmax_t n, unsigned base)
{
	if(n<0)
	{
		kputc('-');
		n = -n;
	}
	kprintu(n, base);
}


static void kputs(const char *str)
{
	while(*str) kputc(*str++);
}


int kprintf(const char *fmt, ...)
{
	const char **args = &fmt+1;
	
	while(*fmt)
	{
		if(*fmt=='%')
		{
			switch(*++fmt)
			{
				case 'd':
					kprintd((intdivmax_t)*args++, 10);
					break;
				case 'u':
					kprintu((uintdivmax_t)*args++, 10);
					break;
				case 'x':
					kprintu((uintdivmax_t)*args++, 16);
					break;
				case 'c':
					kputc((const int)*args++);
					break;
				case 's':
					kputs((const char*)*args++);
					break;
				case '%':
					kputc('%');
					break;
				default:
					return -1;
			}
		}
		else
			kputc(*fmt);
		fmt++;
	}
	return 0;
}

