OUTPUT_FORMAT("elf32-i386")
ENTRY("start")

MEMORY
{
	/* at 0x7c00 is the real mode stack! so only allow this
	 * to go 512B (one sector) less
	 */
	"low_ram" : ORIGIN = 0x500, LENGTH = 0x7a00 - 0x500
	
	/* memory above 1MiB ... but below the potential ISA hole */
	"ext_ram" : ORIGIN = 0x100000, LENGTH = 0xf00000 - 0x100000
}

SECTIONS
{
	. = 0x100000; /* 1MiB */
	".text" :
	{
		*(".text")
		*(".text*")
		*(".text$")
		. = ALIGN(0x200);
	} > "ext_ram" = 0xcc /* 0xf4 */
	
	".data" :
	{
		*(".data")
		*(".data*")
		*(".data$")
		*(".rodata")
		*(".rodata*")
		*(".rodata$")
		. = ALIGN(0x200);
	} > "ext_ram"  = 0xda /* da for data ;) */
	
	/* our .real segment is loaded here (after the data segment)
	 * but we relocated and hence it will reside at 0x500
	 */
	"real_load" = .;
	".real" :
	{
		"real_start" = .;
		*(".real")
		*(".real*")
		*(".real$")
		. = ALIGN(0x200);
	} > "low_ram" AT> "ext_ram" = 0xaa
	"real_size" = SIZEOF(".real");
	. = "real_load" + "real_size";
	
	/* for the multiboot a.out kludge */
	"data_end" = .;
	
	/* the bss segment can not start right after our data because
	 * then the not relocated .real segment would be zero filled by
	 * the Multiboot boot loader, so we must leave the 
	 * .real segment also in ext_ram so we can relocate it, etc.
	 * FIXME: GRUB is picky about nulling this so we must ALIGN(0x1000)
	 * FIXME: ld warns about "changing start of .bss" due to this ALIGN
	 */
	".bss" ABSOLUTE("data_end") : AT("data_end") ALIGN(0x1000)
	{
		*(".bss")
		*(".bss*")
		*(".bss$")
		*(COMMON)
	} >"ext_ram" = 0xff
	
	/DISCARD/ :
	{
		*(".note")
		*(".note*")
		*(".comment")
	}
	
	/* for the multiboot a.out kludge */
	"bss_end" = .;
}

