#define ASM_FILE 1
#include "multiboot.h"

#define MULTIBOOT_HEADER_FLAGS 0 // | MULTIBOOT_AOUT_KLUDGE

#define STK_SZ 0x10000 /* 64KiB */

	.section .bss
	.align 4
	.comm stk, STK_SZ

	.section .text
	.extern memcpy
	.global start, _start

start:
_start:
	cli
	jmp boot

	.align 4
mboot:
	.long MULTIBOOT_HEADER_MAGIC
	.long MULTIBOOT_HEADER_FLAGS
	.long -(MULTIBOOT_HEADER_MAGIC + MULTIBOOT_HEADER_FLAGS)
	/* AOUT KLUDGE */
	#if MULTIBOOT_HEADER_FLAGS & MULTIBOOT_AOUT_KLUDGE
	.long mboot
	.long start
	.long data_end
	.long bss_end
	.long boot /* ip of where to start */
	#endif

boot:
	cli
	movl $(stk + STK_SZ), %esp

	pushl $0
	popf

	cld

	/* push multiboot args */
	push %ebx
	push %eax

	/* disable NMIs */
	inb $0x70, %al
	or $0x80, %al
	outb %al, $0x70

	lidt (idtd)
	lgdt (gdtd)
	ljmp $0x08, $(1f)
1:
	mov $0x10, %eax
	mov %eax, %ds
	mov %eax, %es
	mov %eax, %fs
	mov %eax, %gs
	mov %eax, %ss

	call kmain

1:
	cli
	hlt
	jmp 1b

.align 4
idtd:
	.word 0x3ff
	.long 0

.align 4
gdt:
	/* Figure 3-8. Segment Descriptor [1] */
	/* .byte limit[0:7],limit[8:15],base[0:7],base[8:15],
	 *       base[16:32], present[0]<<7|DPL[0:1]<<5|iscodedataseg[0]<<4|
	 *       type[0:3],
	 *       granularity[0]<<7|32bit[0]<<6|64bit[0]<<5|avail[0]<<4|
	 *       limit[16:19], base[24:31]
	 */
	/* null */
	.byte 0, 0, 0, 0, 0, 0, 0, 0
	
	/* protected mode */
	/* code 0x08 */
	.byte 0xff, 0xff, 0, 0, 0, 1<<7 | 1<<4 | 0xa, 1<<7 | 1<<6 | 0xf, 0x00
	/* data 0x10 */
	.byte 0xff, 0xff, 0, 0, 0, 1<<7 | 1<<4 | 0x2, 1<<7 | 1<<6 | 0xf, 0x00
	
	/* real mode */
	/* code 0x18 */
	.byte 0xff, 0xff, 0, 0, 0, 1<<7 | 1<<4 | 0xa, 0, 0x00
	/* data 0x20 */
	.byte 0xff, 0xff, 0, 0, 0, 1<<7 | 1<<4 | 0x2, 0, 0x00
gdt_end:

.align 4
gdtd:
	.word gdt_end - gdt - 1
	.long gdt

/*
References:
[1] Intel(R) 64 and IA-32 Architectures Software Developer’s Manual:
    Volume 3 (3A, 3B & 3C): System Programming Guide,
    Order Number: 325384-044US, August 2012
*/


