import collections
from .Sampler import Sampler
from .Block import Block

class Blocks(object):
	def __init__(self):
		self._maxblocks = 16
		self._blocks = [ ]
		self._duplicates = collections.defaultdict(list)

	@staticmethod
	def _equal(b1, b2):
		hweight = (Block([ (x ^ y) for (x, y) in zip(b1, b2)])).avghammingweight()
		return hweight < 1.5

	def addblock(self, newblock):
		for (index, refblock) in enumerate(self._blocks):
			if self._equal(refblock, newblock):
				self._duplicates[index].append(newblock)
				return index
		if len(self._blocks) >= self._maxblocks:
			return -1

		self._blocks.append(newblock)
		index = len(self._blocks) - 1
		self._duplicates[index].append(newblock)
#		print(len(self._duplicates))
		return index		

	@property
	def cnt(self):
		return len(self._blocks)

	def getmaxlhcrumb(self, index):
		crumbs = self._duplicates[index]
		sampler = Sampler()
		for sample in crumbs:
			sampler.add(sample)
		return sampler.getmaxlh()

	def maxblock(self):
		max_index = 0
		max_cnt = 0
		for (index, blocks) in self._duplicates.items():
			if len(blocks) > max_cnt:
				max_cnt = len(blocks)
				max_index = index
		return self._duplicates[max_index]


class ExactBlocks(object):
	def __init__(self):
		self._blocks = { }

	def addblock(self, newblock):
		if newblock not in self._blocks:
			self._blocks[newblock] = len(self._blocks)
		return self._blocks[newblock]

	@property
	def cnt(self):
		return len(self._blocks)
